/**
 * Copyright (c) 2024 Enzien Audio, Ltd.
 * 
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 * 
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions, and the following disclaimer.
 * 
 * 2. Redistributions in binary form must reproduce the phrase "powered by heavy",
 *    the heavy logo, and a hyperlink to https://enzienaudio.com, all in a visible
 *    form.
 * 
 *   2.1 If the Application is distributed in a store system (for example,
 *       the Apple "App Store" or "Google Play"), the phrase "powered by heavy"
 *       shall be included in the app description or the copyright text as well as
 *       the in the app itself. The heavy logo will shall be visible in the app
 *       itself as well.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 * SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER
 * CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF
 * THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * 
 */

#ifndef _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
#define _HEAVY_CONTEXT_WSTD_CRSHR_HPP_

// object includes
#include "HeavyContext.hpp"
#include "HvControlVar.h"
#include "HvMath.h"
#include "HvSignalLine.h"
#include "HvSignalVar.h"
#include "HvControlCast.h"
#include "HvControlBinop.h"

class Heavy_WSTD_CRSHR : public HeavyContext {

 public:
  Heavy_WSTD_CRSHR(double sampleRate, int poolKb=10, int inQueueKb=2, int outQueueKb=0);
  ~Heavy_WSTD_CRSHR();

  const char *getName() override { return "WSTD_CRSHR"; }
  int getNumInputChannels() override { return 2; }
  int getNumOutputChannels() override { return 2; }

  int process(float **inputBuffers, float **outputBuffer, int n) override;
  int processInline(float *inputBuffers, float *outputBuffer, int n) override;
  int processInlineInterleaved(float *inputBuffers, float *outputBuffer, int n) override;

  int getParameterInfo(int index, HvParameterInfo *info) override;
  struct Parameter {
    struct In {
      enum ParameterIn : hv_uint32_t {
        CRSHR = 0xE2DA4DC3, // Crshr
        MIX = 0x97D46AC6, // Mix
      };
    };
  };

 private:
  HvTable *getTableForHash(hv_uint32_t tableHash) override;
  void scheduleMessageForReceiver(hv_uint32_t receiverHash, HvMessage *m) override;

  // static sendMessage functions
  static void cBinop_T3cFiFco_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_zumpO4aG_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_1S3Np4tx_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Sar6jsKn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_hqlBriIX_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_fuMVx7aD_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_Q83OiJPh_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_ifPF59ME_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_4INoUTWo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_WghLkSVv_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_tzC0LXAn_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_v6qpipyf_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cVar_Y3YcY1kA_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cCast_qu9uNVPy_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_2oCj1SzR_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_J8ClA6gQ_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cSend_vyYtMN2A_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_OPzBGkfo_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_8ONdNMCm_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_0HYevUBU_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cMsg_aPMQe52E_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cBinop_LYBZ8yx1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_LJXY8Vrb_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_0upA7yPL_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_wWGuLTo1_sendMessage(HeavyContextInterface *, int, const HvMessage *);
  static void cReceive_ysZHiqHI_sendMessage(HeavyContextInterface *, int, const HvMessage *);

  // objects
  SignalLine sLine_zYmXLVqY;
  SignalLine sLine_bNUuLRA0;
  SignalLine sLine_nN09He6Q;
  ControlBinop cBinop_T3cFiFco;
  ControlBinop cBinop_zumpO4aG;
  ControlBinop cBinop_1S3Np4tx;
  ControlBinop cBinop_Sar6jsKn;
  ControlBinop cBinop_Q83OiJPh;
  ControlBinop cBinop_4INoUTWo;
  ControlBinop cBinop_WghLkSVv;
  ControlBinop cBinop_tzC0LXAn;
  ControlVar cVar_v6qpipyf;
  ControlVar cVar_Y3YcY1kA;
  SignalVarf sVarf_cPSbmbw2;
  SignalVarf sVarf_CAmsfWGD;
  SignalVarf sVarf_77oX1CYx;
  SignalVarf sVarf_WX0FbuiC;
  ControlBinop cBinop_OPzBGkfo;
  ControlBinop cBinop_LYBZ8yx1;
};

#endif // _HEAVY_CONTEXT_WSTD_CRSHR_HPP_
